@@ -274,8 +274,8 @@ def main():
             for authority in authorities:
                 output = f"Authority:  {authority}"
                 output += f"\nSource:     {authority.template}"
-                if authority.URL:
-                    output += "\nURL:        " + authority.URL
+                if authority.base_citation().URL:
+                    output += "\nURL:        " + authority.base_citation().URL
                 output += f"\nReferences: {len(authority.citations)}"
                 outputs.append(output)
             print('\n\n'.join(outputs))
@@ -319,12 +319,12 @@ def main():
         # pretty-print citation information by indenting each bit of info
         # based on the length of the longest token name
         key_lengths = [
-            len(k) for k in citation.processed_tokens.keys()
-            if citation.processed_tokens[k]
+            len(k) for k in citation.tokens.keys()
+            if citation.tokens[k]
         ]
         tab_width = max(key_lengths) + 2
         print('Source: '.ljust(tab_width) + str(citation.template))
-        for key, value in citation.processed_tokens.items():
+        for key, value in citation.tokens.items():
             if not value:
                 continue
             print(
